@@ -29,6 +29,8 @@ import java.math.BigDecimal;
 public class CatalogDoorOrWindow extends CatalogPieceOfFurniture implements DoorOrWindow {
   private final float   wallThickness;
   private final float   wallDistance;
+  private final boolean wallCutOutOnBothSides;
+  private final boolean widthDepthDeformable;
   private final Sash [] sashes;
   private final String  cutOutShape;
 
@@ -363,13 +365,71 @@ public class CatalogDoorOrWindow extends CatalogPieceOfFurniture implements Door
                              float [][] modelRotation, boolean backFaceShown, String creator, 
                              boolean resizable, boolean deformable, boolean texturable, 
                              BigDecimal price, BigDecimal valueAddedTaxPercentage, String currency) {
+    this(id, name, description, information, tags, creationDate, grade, 
+        icon, planIcon, model, width, depth, height, elevation, dropOnTopElevation, movable, 
+        cutOutShape, wallThickness, wallDistance, true, true, sashes,
+        modelRotation, backFaceShown, null, creator, resizable, deformable, texturable, price, valueAddedTaxPercentage, currency);  
+  }
+         
+  /**
+   * Creates an unmodifiable catalog door or window of the default catalog.
+   * @param id    the id of the new door or window, or <code>null</code>
+   * @param name  the name of the new door or window
+   * @param description the description of the new door or window 
+   * @param information additional information associated to the new door or window
+   * @param tags tags associated to the new door or window
+   * @param creationDate creation date of the new door or window in milliseconds since the epoch 
+   * @param grade grade of the new door or window or <code>null</code>
+   * @param icon content of the icon of the new door or window
+   * @param planIcon content of the icon of the new piece displayed in plan
+   * @param model content of the 3D model of the new door or window
+   * @param width  the width in centimeters of the new door or window
+   * @param depth  the depth in centimeters of the new door or window
+   * @param height  the height in centimeters of the new door or window
+   * @param elevation  the elevation in centimeters of the new door or window
+   * @param dropOnTopElevation a percentage of the height at which should be placed 
+   *            an object dropped on the new piece
+   * @param movable if <code>true</code>, the new door or window is movable
+   * @param cutOutShape the shape used to cut out walls that intersect the new door or window
+   * @param wallThickness a value in percentage of the depth of the new door or window
+   * @param wallDistance a distance in percentage of the depth of the new door or window
+   * @param wallCutOutOnBothSides  if <code>true</code> the new door or window should cut out 
+   *            the both sides of the walls it intersects
+   * @param widthDepthDeformable if <code>false</code>, the width and depth of the new door or window may 
+   *            not be changed independently from each other
+   * @param sashes the sashes attached to the new door or window
+   * @param modelRotation the rotation 3 by 3 matrix applied to the door or window model
+   * @param backFaceShown <code>true</code> if back face should be shown instead of front faces
+   * @param modelSize size of the 3D model of the new light
+   * @param creator the creator of the model
+   * @param resizable if <code>true</code>, the size of the new door or window may be edited
+   * @param deformable if <code>true</code>, the width, depth and height of the new piece may 
+   *            change independently from each other
+   * @param texturable if <code>false</code> this piece should always keep the same color or texture.
+   * @param price the price of the new door or window, or <code>null</code> 
+   * @param valueAddedTaxPercentage the Value Added Tax percentage applied to the 
+   *             price of the new door or window or <code>null</code>
+   * @param currency the price currency, noted with ISO 4217 code, or <code>null</code> 
+   * @since 5.5
+   */
+  public CatalogDoorOrWindow(String id, String name, String description, 
+                             String information, String [] tags, Long creationDate, Float grade, 
+                             Content icon, Content planIcon, Content model, 
+                             float width, float depth, float height, float elevation, float dropOnTopElevation, boolean movable, 
+                             String cutOutShape, float wallThickness, float wallDistance, 
+                             boolean wallCutOutOnBothSides, boolean widthDepthDeformable, Sash [] sashes,
+                             float [][] modelRotation, boolean backFaceShown, Long modelSize, String creator, 
+                             boolean resizable, boolean deformable, boolean texturable, 
+                             BigDecimal price, BigDecimal valueAddedTaxPercentage, String currency) {
     super(id, name, description, information, tags, creationDate, grade, 
         icon, planIcon, model, width, depth, height, elevation, dropOnTopElevation, movable, 
-        null, modelRotation, backFaceShown, creator, resizable, deformable, texturable, 
+        null, modelRotation, backFaceShown, modelSize, creator, resizable, deformable, texturable, false,
         price, valueAddedTaxPercentage, currency);
     this.cutOutShape = cutOutShape;
     this.wallThickness = wallThickness;
     this.wallDistance = wallDistance;
+    this.wallCutOutOnBothSides = wallCutOutOnBothSides;
+    this.widthDepthDeformable = widthDepthDeformable;
     this.sashes = sashes;
   }
          
@@ -399,10 +459,45 @@ public class CatalogDoorOrWindow extends CatalogPieceOfFurniture implements Door
                              float wallThickness, float wallDistance, Sash [] sashes, 
                              Integer color, float [][] modelRotation, boolean backFaceShown, 
                              float iconYaw, boolean proportional) {
-    super(name, icon, model, width, depth, height, elevation, movable,   
-        color, modelRotation, backFaceShown, iconYaw, proportional);
+    this(name, icon, model, width, depth, height, elevation, movable,   
+        wallThickness, wallDistance, sashes, color, modelRotation, backFaceShown, null, null, iconYaw, proportional);
+  }
+
+  /**
+   * Creates a modifiable catalog door or window with all its values.
+   * @param name  the name of the new door or window
+   * @param icon content of the icon of the new door or window
+   * @param model content of the 3D model of the new door or window
+   * @param width  the width in centimeters of the new door or window
+   * @param depth  the depth in centimeters of the new door or window
+   * @param height  the height in centimeters of the new door or window
+   * @param elevation  the elevation in centimeters of the new door or window
+   * @param movable if <code>true</code>, the new door or window is movable
+   * @param wallThickness a value in percentage of the depth of the new door or window
+   * @param wallDistance a distance in percentage of the depth of the new door or window
+   * @param sashes the sashes attached to the new door or window
+   * @param color the color of the door or window as RGB code or <code>null</code> 
+   *        if door or window color is unchanged
+   * @param modelRotation the rotation 3 by 3 matrix applied to the door or window model
+   * @param backFaceShown <code>true</code> if back face should be shown
+   * @param modelSize size of the 3D model of the new piece
+   * @param creator the creator of the model
+   * @param iconYaw the yaw angle used to create the door or window icon
+   * @param proportional if <code>true</code>, size proportions will be kept
+   * @since 5.5
+   */
+  public CatalogDoorOrWindow(String name, Content icon, Content model, 
+                             float width, float depth, float height,
+                             float elevation, boolean movable, 
+                             float wallThickness, float wallDistance, Sash [] sashes, 
+                             Integer color, float [][] modelRotation, boolean backFaceShown, Long modelSize, 
+                             String creator, float iconYaw, boolean proportional) {
+    super(name, icon, model, width, depth, height, elevation, movable, null,   
+        color, modelRotation, backFaceShown, modelSize, creator, iconYaw, proportional);
     this.wallThickness = wallThickness;
     this.wallDistance = wallDistance;
+    this.wallCutOutOnBothSides = true;
+    this.widthDepthDeformable = true;
     this.sashes = sashes;
     this.cutOutShape = null;
   }
@@ -422,6 +517,24 @@ public class CatalogDoorOrWindow extends CatalogPieceOfFurniture implements Door
   public float getWallDistance() {
     return this.wallDistance;
   }
+
+  /**
+   * Returns <code>true</code> if this door or window should cut out the both sides
+   * of the walls it intersects, even if its front or back side are within the wall thickness. 
+   * @since 5.5 
+   */
+  public boolean isWallCutOutOnBothSides() {
+    return this.wallCutOutOnBothSides;
+  }
+
+  /**
+   * Returns <code>false</code> if the width and depth of the new door or window may 
+   * not be changed independently from each other.
+   * @since 5.5
+   */
+  public boolean isWidthDepthDeformable() {
+    return this.widthDepthDeformable;
+  }
   
   /**
    * Returns a copy of the sashes attached to this door or window.
