@@ -45,19 +45,22 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
   private static final long serialVersionUID = 1L;
   
   private static final double TWICE_PI = 2 * Math.PI;
+  private static final double STRAIGHT_WALL_ANGLE_MARGIN  = Math.toRadians(1);
+  private static final double ROUND_WALL_ANGLE_MARGIN     = Math.toRadians(10);
   
   /**
    * The properties of a piece of furniture that may change. <code>PropertyChangeListener</code>s added 
    * to a piece of furniture will be notified under a property name equal to the string value of one these properties.
    */
   public enum Property {NAME, NAME_VISIBLE, NAME_X_OFFSET, NAME_Y_OFFSET, NAME_STYLE, NAME_ANGLE,
-      DESCRIPTION, PRICE, WIDTH, DEPTH, HEIGHT, COLOR, TEXTURE, MODEL_MATERIALS, SHININESS, VISIBLE, X, Y, ELEVATION, ANGLE, MODEL_MIRRORED, MOVABLE, LEVEL};
+      DESCRIPTION, PRICE, WIDTH, WIDTH_IN_PLAN, DEPTH, DEPTH_IN_PLAN, HEIGHT, HEIGHT_IN_PLAN, 
+      COLOR, TEXTURE, MODEL_MATERIALS, SHININESS, VISIBLE, X, Y, ELEVATION, ANGLE, PITCH, ROLL, MODEL_MIRRORED, MOVABLE, LEVEL};
   
   /** 
    * The properties on which home furniture may be sorted.  
    */
   public enum SortableProperty {CATALOG_ID, NAME, WIDTH, DEPTH, HEIGHT, MOVABLE, 
-                                DOOR_OR_WINDOW, COLOR, TEXTURE, VISIBLE, X, Y, ELEVATION, ANGLE,
+                                DOOR_OR_WINDOW, COLOR, TEXTURE, VISIBLE, X, Y, ELEVATION, ANGLE, MODEL_SIZE, CREATOR, 
                                 PRICE, VALUE_ADDED_TAX, VALUE_ADDED_TAX_PERCENTAGE, PRICE_VALUE_ADDED_TAX_INCLUDED, LEVEL};
   private static final Map<SortableProperty, Comparator<HomePieceOfFurniture>> SORTABLE_PROPERTY_COMPARATORS;
   private static final float [][] IDENTITY = new float [][] {{1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
@@ -168,6 +171,36 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
           return HomePieceOfFurniture.compare(piece1.angle, piece2.angle);
         }
       });
+    SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.MODEL_SIZE, new Comparator<HomePieceOfFurniture>() {
+        public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
+          Long piece1ModelSize = HomePieceOfFurniture.getComparableModelSize(piece1);
+          Long piece2ModelSize = HomePieceOfFurniture.getComparableModelSize(piece2);
+          if (piece1ModelSize == piece2ModelSize) {
+            return 0; 
+          } else if (piece1ModelSize == null) {
+            return -1;
+          } else if (piece2ModelSize == null) {
+            return 1; 
+          } else {
+            return piece1ModelSize < piece2ModelSize 
+                ? -1 
+                : (piece1ModelSize == piece2ModelSize ? 0 : 1);
+          }
+        }
+      });
+    SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.CREATOR, new Comparator<HomePieceOfFurniture>() {
+        public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
+          if (piece1.creator == piece2.creator) {
+            return 0;
+          } else if (piece1.creator == null) {
+            return -1;
+          } else if (piece2.creator == null) {
+            return 1; 
+          } else {
+            return collator.compare(piece1.creator, piece2.creator);
+          }
+        }
+      });
     SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.LEVEL, new Comparator<HomePieceOfFurniture>() {
         public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
           return HomePieceOfFurniture.compare(piece1.getLevel(), piece2.getLevel());
@@ -229,6 +262,23 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
     }
   }
   
+  private static Long getComparableModelSize(HomePieceOfFurniture piece) {
+    if (piece instanceof HomeFurnitureGroup) {
+      Long biggestModelSize = null;
+      for (HomePieceOfFurniture childPiece : ((HomeFurnitureGroup)piece).getFurniture()) {
+        Long modelSize = getComparableModelSize(childPiece);
+        if (modelSize != null
+            && (biggestModelSize == null
+                || biggestModelSize.longValue() < modelSize.longValue())) {
+          biggestModelSize = modelSize;
+        }
+      }
+      return biggestModelSize;
+    } else {
+      return piece.modelSize;
+    }
+  }
+  
   private String                 catalogId;
   private String                 name;
   private boolean                nameVisible;
@@ -241,9 +291,13 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
   private Content                icon;
   private Content                planIcon;
   private Content                model;
+  private Long                   modelSize;                    
   private float                  width;
+  private float                  widthInPlan;
   private float                  depth;
+  private float                  depthInPlan;
   private float                  height;
+  private float                  heightInPlan;
   private float                  elevation;
   private float                  dropOnTopElevation;
   private boolean                movable;
@@ -253,12 +307,14 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
   private HomeTexture            texture;
   private Float                  shininess;
   private float [][]             modelRotation;
+  private boolean                modelCenteredAtOrigin;
   private String                 staircaseCutOutShape;
   private String                 creator;
   private boolean                backFaceShown;
   private boolean                resizable;
   private boolean                deformable;
   private boolean                texturable;
+  private boolean                horizontallyRotatable;
   private BigDecimal             price;
   private BigDecimal             valueAddedTaxPercentage;
   private String                 currency;
@@ -266,12 +322,15 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
   private float                  x;
   private float                  y;
   private float                  angle;
+  private float                  pitch;
+  private float                  roll;
   private boolean                modelMirrored;
   private Level                  level;
   
   private transient PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
   private transient Shape shapeCache;
 
+
   /**
    * Creates a home piece of furniture from an existing piece.
    * @param piece the piece from which data are copied
@@ -283,6 +342,7 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
     this.icon = piece.getIcon();
     this.planIcon = piece.getPlanIcon();
     this.model = piece.getModel();
+    this.modelSize = piece.getModelSize();
     this.width = piece.getWidth();
     this.depth = piece.getDepth();
     this.height = piece.getHeight();
@@ -298,12 +358,12 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
     this.resizable = piece.isResizable();
     this.deformable = piece.isDeformable();
     this.texturable = piece.isTexturable();
+    this.horizontallyRotatable = piece.isHorizontallyRotatable();
     this.price = piece.getPrice();
     this.valueAddedTaxPercentage = piece.getValueAddedTaxPercentage();
     this.currency = piece.getCurrency();
     if (piece instanceof HomePieceOfFurniture) {
-      HomePieceOfFurniture homePiece = 
-          (HomePieceOfFurniture)piece;
+      HomePieceOfFurniture homePiece = (HomePieceOfFurniture)piece;
       this.catalogId = homePiece.getCatalogId();
       this.nameVisible = homePiece.isNameVisible();
       this.nameXOffset = homePiece.getNameXOffset();
@@ -311,7 +371,13 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
       this.nameAngle = homePiece.getNameAngle();
       this.nameStyle = homePiece.getNameStyle();
       this.visible = homePiece.isVisible();
+      this.widthInPlan = homePiece.getWidthInPlan();
+      this.depthInPlan = homePiece.getDepthInPlan();
+      this.heightInPlan = homePiece.getHeightInPlan();
+      this.modelCenteredAtOrigin = homePiece.isModelCenteredAtOrigin();
       this.angle = homePiece.getAngle();
+      this.pitch = homePiece.getPitch();
+      this.roll = homePiece.getRoll();
       this.x = homePiece.getX();
       this.y = homePiece.getY();
       this.modelMirrored = homePiece.isModelMirrored();
@@ -323,6 +389,10 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
         this.catalogId = ((CatalogPieceOfFurniture)piece).getId();
       }      
       this.visible = true;
+      this.widthInPlan = this.width;
+      this.depthInPlan = this.depth;
+      this.heightInPlan = this.height;
+      this.modelCenteredAtOrigin = true; // false by default for version < 5.5
       this.x = this.width / 2;
       this.y = this.depth / 2;
     }
@@ -338,11 +408,29 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
     this.resizable = true;
     this.deformable = true;
     this.texturable = true;
+    this.horizontallyRotatable = true;
     this.propertyChangeSupport = new PropertyChangeSupport(this);
+    this.widthInPlan =
+    this.depthInPlan =
+    this.heightInPlan = Float.NEGATIVE_INFINITY;
     in.defaultReadObject();
     
     // Ensure angle is always positive and between 0 and 2 PI
     this.angle = (float)((this.angle % TWICE_PI + TWICE_PI) % TWICE_PI);
+    // Update new fields used to store dimensions of a piece in plan after pitch or roll is applied to it
+    if (this.widthInPlan == Float.NEGATIVE_INFINITY
+        || this.depthInPlan == Float.NEGATIVE_INFINITY
+        || this.heightInPlan == Float.NEGATIVE_INFINITY) {
+      this.widthInPlan = this.width;
+      this.depthInPlan = this.depth;
+      this.heightInPlan = this.height;
+      this.pitch = 0;
+      this.roll = 0;
+    }
+    if (!this.modelCenteredAtOrigin) {
+      // Keep default value to false only if model rotation matrix isn't identity
+      this.modelCenteredAtOrigin = Arrays.deepEquals(IDENTITY, this.modelRotation);
+    }
   }
 
   /**
@@ -539,6 +627,28 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
     }
   }
 
+  /**
+   * Returns the depth of this piece of furniture in the horizontal plan (after pitch or roll is applied to it). 
+   * @since 5.5
+   */
+  public float getDepthInPlan() {
+    return this.depthInPlan;
+  }
+
+  /**
+   * Sets the depth of this piece of furniture in the horizontal plan (after pitch or roll is applied to it). 
+   * listeners added to this piece will receive a change notification.
+   * @since 5.5
+   */
+  public void setDepthInPlan(float depthInPlan) {
+    if (depthInPlan != this.depthInPlan) {
+      float oldDepth = this.depthInPlan;
+      this.depthInPlan = depthInPlan;
+      this.shapeCache = null;
+      this.propertyChangeSupport.firePropertyChange(Property.DEPTH_IN_PLAN.name(), oldDepth, depthInPlan);
+    }
+  }
+
   /**
    * Returns the height of this piece of furniture.
    */
@@ -563,6 +673,27 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
     }
   }
 
+  /**
+   * Returns the height of this piece of furniture from the horizontal plan (after pitch or roll is applied to it). 
+   * @since 5.5
+   */
+  public float getHeightInPlan() {
+    return this.heightInPlan;
+  }
+
+  /**
+   * Sets the height of this piece of furniture from the horizontal plan (after pitch or roll is applied to it). 
+   * Once this piece is updated, listeners added to this piece will receive a change notification.
+   * @since 5.5
+   */
+  public void setHeightInPlan(float heightInPlan) {
+    if (heightInPlan != this.heightInPlan) {
+      float oldHeight = this.heightInPlan;
+      this.heightInPlan = heightInPlan;
+      this.propertyChangeSupport.firePropertyChange(Property.HEIGHT_IN_PLAN.name(), oldHeight, heightInPlan);
+    }
+  }
+
   /**
    * Returns the width of this piece of furniture.
    */
@@ -588,6 +719,39 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
     }
   }
 
+  /**
+   * Returns the width of this piece of furniture in the horizontal plan (after pitch or roll is applied to it). 
+   * @since 5.5
+   */
+  public float getWidthInPlan() {
+    return this.widthInPlan;
+  }
+
+  /**
+   * Sets the width of this piece of furniture in the horizontal plan (after pitch or roll is applied to it). 
+   * Once this piece is updated, listeners added to this piece will receive a change notification.
+   * @since 5.5
+   */
+  public void setWidthInPlan(float widthInPlan) {
+    if (widthInPlan != this.widthInPlan) {
+      float oldWidth = this.widthInPlan;
+      this.widthInPlan = widthInPlan;
+      this.shapeCache = null;
+      this.propertyChangeSupport.firePropertyChange(Property.WIDTH_IN_PLAN.name(), oldWidth, widthInPlan);
+    }
+  }
+
+  /**
+   * Scales this piece of furniture with the given <code>scale</code>.
+   * Once this piece is updated, listeners added to this piece will receive a change notification.
+   * @since 5.5
+   */
+  public void scale(float scale) {
+    setWidth(getWidth() * scale);
+    setDepth(getDepth() * scale);
+    setHeight(getHeight() * scale);
+  }
+  
   /**
    * Returns the elevation of the bottom of this piece of furniture on its level. 
    */
@@ -680,6 +844,23 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
     return this.model;
   }
 
+  /**
+   * Sets the size of the 3D model of this piece of furniture.
+   * This method should be called only to update a piece created with an older version.
+   * @since 5.5
+   */
+  public void setModelSize(Long modelSize) {
+    this.modelSize = modelSize;
+  }
+  
+  /**
+   * Returns the size of the 3D model of this piece of furniture.
+   * @since 5.5
+   */
+  public Long getModelSize() {
+    return this.modelSize;
+  }
+
   /**
    * Sets the materials of the 3D model of this piece of furniture. 
    * Once this piece is updated, listeners added to this piece will receive a change notification.
@@ -813,6 +994,14 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
     return this.deformable;    
   }
 
+  /**
+   * Returns <code>true</code> if this piece is deformable.
+   * @since 5.5
+   */
+  public boolean isWidthDepthDeformable() {
+    return isDeformable();
+  }
+  
   /**
    * Returns <code>false</code> if this piece should always keep the same color or texture.
    * @since 3.0
@@ -821,6 +1010,14 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
     return this.texturable;
   }
 
+  /**
+   * Returns <code>false</code> if this piece should not rotate around an horizontal axis.
+   * @since 5.5
+   */
+  public boolean isHorizontallyRotatable() {
+    return this.horizontallyRotatable;
+  }
+  
   /**
    * Returns the price of this piece of furniture or <code>null</code>. 
    */
@@ -940,7 +1137,7 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
   }
 
   /**
-   * Returns the angle in radians of this piece of furniture around vertical axis. 
+   * Returns the angle in radians of this piece around vertical axis. 
    */
   public float getAngle() {
     return this.angle;
@@ -961,6 +1158,70 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
     }
   }
 
+  /**
+   * Returns the pitch angle in radians of this piece of furniture.
+   * @since 5.5
+   */
+  public float getPitch() {
+    return this.pitch;
+  }
+
+  /**
+   * Sets the pitch angle in radians of this piece and notifies listeners of this change.
+   * Pitch axis is horizontal lateral (or transverse) axis. 
+   * @since 5.5
+   */
+  public void setPitch(float pitch) {
+    if (isHorizontallyRotatable()) {
+      // Ensure pitch is always positive and between 0 and 2 PI
+      pitch = (float)((pitch % TWICE_PI + TWICE_PI) % TWICE_PI);
+      if (pitch != this.pitch) {
+        float oldPitch = this.pitch;
+        this.pitch = pitch;
+        this.shapeCache = null;
+        this.propertyChangeSupport.firePropertyChange(Property.PITCH.name(), oldPitch, pitch);
+      }
+    } else {
+      throw new IllegalStateException("Piece can't be rotated around an horizontal axis");
+    }
+  }
+
+  /**
+   * Returns the roll angle in radians of this piece of furniture.
+   * @since 5.5
+   */
+  public float getRoll() {
+    return this.roll;
+  }
+
+  /**
+   * Sets the roll angle in radians of this piece and notifies listeners of this change.
+   * Roll axis is horizontal longitudinal axis. 
+   * @since 5.5
+   */
+  public void setRoll(float roll) {
+    if (isHorizontallyRotatable()) {
+      // Ensure roll is always positive and between 0 and 2 PI
+      roll = (float)((roll % TWICE_PI + TWICE_PI) % TWICE_PI);
+      if (roll != this.roll) {
+        float oldRoll = this.roll;
+        this.roll = roll;
+        this.shapeCache = null;
+        this.propertyChangeSupport.firePropertyChange(Property.ROLL.name(), oldRoll, roll);
+      }
+    } else {
+      throw new IllegalStateException("Piece can't be rotated around an horizontal axis");
+    }
+  }
+  
+  /**
+   * Returns <code>true</code> if the pitch or roll angle of this piece is different from 0. 
+   * @since 5.5
+   */
+  public boolean isHorizontallyRotated() {
+    return this.roll != 0 || this.pitch != 0;
+  }
+
   /**
    * Returns <code>true</code> if the model of this piece should be mirrored.
    */
@@ -996,6 +1257,27 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
                            {this.modelRotation[2][0], this.modelRotation[2][1], this.modelRotation[2][2]}};
   }
 
+  /**
+   * Sets whether model center should be always centered at the origin
+   * when model rotation isn't <code>null</code>.
+   * This method should be called only to keep unchanged the (wrong) location 
+   * of a rotated model created with version < 5.5.  
+   * @since 5.5  
+   */
+  public void setModelCenteredAtOrigin(boolean modelCenteredAtOrigin) {
+    this.modelCenteredAtOrigin = modelCenteredAtOrigin;
+  }
+  
+  /**
+   * Returns <code>true</code> if model center should be always centered at the origin
+   * when model rotation isn't <code>null</code>.
+   * @return <code>false</code> by default if version < 5.5
+   * @since 5.5  
+   */
+  public boolean isModelCenteredAtOrigin() {
+    return this.modelCenteredAtOrigin;
+  }
+  
   /**
    * Returns the shape used to cut out upper levels when they intersect with the piece   
    * like a staircase.
@@ -1191,15 +1473,42 @@ public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture
   }
 
   /**
-   * Returns the shape matching this piece.
+   * Returns <code>true</code> if the front side of this piece is parallel to the given <code>wall</code> 
+   * with a margin.
+   * @since 5.5
+   */
+  public boolean isParallelToWall(Wall wall) {
+    if (wall.getArcExtent() == null) {
+      float deltaY = wall.getYEnd() - wall.getYStart();
+      float deltaX = wall.getXEnd() - wall.getXStart();
+      if (deltaX == 0 && deltaY == 0) {
+        return false;
+      } else {
+        // Check parallelism with line joining wall ends  
+        double wallAngle = Math.atan2(deltaY, deltaX); 
+        double pieceWallAngle = Math.abs(wallAngle - getAngle()) % Math.PI;
+        return pieceWallAngle <= STRAIGHT_WALL_ANGLE_MARGIN || (Math.PI - pieceWallAngle) <= STRAIGHT_WALL_ANGLE_MARGIN;
+      }
+    } else {
+      // Tangent angle at piece center
+      double tangentAngle = Math.PI / 2 + Math.atan2(
+          wall.getYArcCircleCenter() - getY(), wall.getXArcCircleCenter() - getX()); 
+      double pieceWallAngle = Math.abs(tangentAngle - getAngle()) % Math.PI;
+      // Be more tolerant for angles along round walls 
+      return pieceWallAngle <= ROUND_WALL_ANGLE_MARGIN || (Math.PI - pieceWallAngle) <= ROUND_WALL_ANGLE_MARGIN;
+    }
+  }
+  
+  /**
+   * Returns the shape matching this piece in the horizontal plan.
    */
   private Shape getShape() {
     if (this.shapeCache == null) {
       // Create the rectangle that matches piece bounds
       Rectangle2D pieceRectangle = new Rectangle2D.Float(
-          getX() - getWidth() / 2,
-          getY() - getDepth() / 2,
-          getWidth(), getDepth());
+          getX() - getWidthInPlan() / 2,
+          getY() - getDepthInPlan() / 2,
+          getWidthInPlan(), getDepthInPlan());
       // Apply rotation to the rectangle
       AffineTransform rotation = AffineTransform.getRotateInstance(getAngle(), getX(), getY());
       PathIterator it = pieceRectangle.getPathIterator(rotation);
